from pylib_0xe.file.file import File
from pylib_0xe.path.path_helper import PathHelper


class Templates:
    @staticmethod
    def read(*args: str) -> str:
        return File.read_file(PathHelper.from_root(__file__, *args))



from ....helpers.config.local_config import LocalConfig
from ....models.provider import Provider


class ProviderFinder:
    @staticmethod
    def by_name(name: str) -> Provider:
        filtered = ProviderFinder.filter_providers(
            lambda provider: name == provider.name
        )
        return ProviderFinder.get_single_provider(filtered, name)

    @staticmethod
    def by_abbreviation(abbreviation: str) -> Provider:
        filtered = ProviderFinder.filter_providers(
            lambda provider: abbreviation in provider.abbreviations
        )
        return ProviderFinder.get_single_provider(filtered, abbreviation)

    @staticmethod
    def all() -> list[Provider]:
        providers = LocalConfig.read("providers")
        return [
            Provider.from_dict({"name": provider, **properties})
            for provider, properties in providers.items()
        ]

    @staticmethod
    def default() -> Provider:
        problem_provider = LocalConfig.read("problem.provider")
        return ProviderFinder.by_abbreviation(problem_provider)

    @staticmethod
    def filter_providers(predicate) -> list[Provider]:
        return list(filter(predicate, ProviderFinder.all()))

    @staticmethod
    def get_single_provider(filtered: list[Provider], search_criteria: str) -> Provider:
        if not filtered:
            raise Exception(f"No valid provider found by {search_criteria}")
        return filtered[0]



from __future__ import annotations
import os
from typing import Any
from dataclasses import dataclass, field
from ...helpers.config.config import Config
from pylib_0xe.file.file import File
from pylib_0xe.json.json_helper import JsonHelper


@dataclass
class LocalConfig:
    filename: str
    home_path: list[str] = field(
        default_factory=lambda: [os.path.normpath(os.path.abspath(os.sep))]
    )

    def build(
        self,
    ) -> LocalConfig:
        found = False
        path = os.getcwd()
        try:
            while not found:
                if path in self.home_path:
                    raise Exception
                found = True
                try:
                    self.json = File.read_json(f"{self.filename}.json")
                except Exception:
                    self.filename = os.path.join("..", self.filename)
                    path = os.path.normpath(os.path.join(path, ".."))
                    found = False
        except Exception:
            print(
                "Please provide a local config file by running\
                'contest-builder --init' command"
            )
            exit(0)
        return self

    def get(self, selector: str = "", default: Any = None) -> Any:
        value = JsonHelper.selector_get_value(self.json, selector)
        if value != {}:
            return value
        return default

    @staticmethod
    def read(selector: str) -> Any:
        filename = Config.read("main.config.local")
        return LocalConfig(filename).build().get(selector)



from ...helpers.terminal.arg_helper import ArgHelper


class CatHelper:
    @staticmethod
    def meow():
        helper = (
            ArgHelper(["name", "problem", "run", "init"])
            .check_help()
            .check_help("elp")
            .check_args()
        )
        if len(helper.get_data()) == 0:
            helper.show_valid_options().done()



from pylib_0xe.config.config import Config as CFG
from typing import Any


class Config(CFG):
    @staticmethod
    def read(selector: str, **kwargs) -> Any:
        config = CFG(file_path=__file__)
        return config.read(selector, **kwargs)



from __future__ import annotations
from ..helpers.config.local_config import LocalConfig
from ..helpers.file.file_helper import FileHelper
from ..helpers.config.templates import Templates
from ..helpers.config.config import Config


class InitMediator:
    def read_configs(self) -> InitMediator:
        self.name = Config.read("main.config.local")
        self.template_config = Templates.read("templates", "template.json")
        return self

    def create_local_config(self) -> InitMediator:
        FileHelper.write_file(f"{self.name}.json", file=self.template_config)
        return self

    def create_templates(self) -> InitMediator:
        templates = LocalConfig.read("templates")
        for _, obj in templates.items():
            for _, path in obj.items():
                FileHelper.write_file(*path, file=Templates.read(*path))
        return self

    def closure(self) -> InitMediator:
        print(f"Succesfully generated {self.name} and templates")
        return self



import os


class FolderHelper:
    @staticmethod
    def create_path(*args: str) -> None:
        if len(args) == 0:
            return
        path = os.path.join(*args)
        try:
            os.makedirs(path, exist_ok=True)
        except Exception as e:
            print(f"Error: {e}")



from __future__ import annotations
from dataclasses import dataclass, field
from typing import List
from pylib_0xe.data.data_transfer_object import DataTransferObject


@dataclass
class Language(DataTransferObject):
    """Represents a programming language."""

    name: str = ""
    abbreviations: List[str] = field(default_factory=list)
    extensions: List[str] = field(default_factory=list)

    @staticmethod
    def name_mapper(name: str) -> str:
        """Map the language name to lowercase."""
        return name.lower()

    @staticmethod
    def extensions_mapper(ext_list: List[str]) -> List[str]:
        """Map each extension to lowercase."""
        return [ext.lower() for ext in ext_list]

    @property
    def ext(self) -> str:
        """Get the first extension in lowercase."""
        return self.extensions[0].lower() if self.extensions else ""

    @ext.setter
    def ext(self, value: str) -> None:
        """Set the first extension to the specified value."""
        if value in self.extensions:
            while self.extensions[0] != value:
                self.extensions = [*self.extensions[1:], self.extensions[0]]

    def __eq__(self, other: object) -> bool:
        """Check if two Language objects are equal."""
        if not isinstance(other, Language):
            return False
        return self.name == other.name



import time


class Timer:
    def __init__(self) -> None:
        self.timer = time.time() * 1000

    def elapsed_ms(self) -> float:
        return time.time() * 1000 - self.timer

    def timestamp(self) -> str:
        return f"{self.elapsed_ms():03f}ms"



import re
from ...types.name_types import NameTypes


class ProblemHelper:
    @staticmethod
    def problem_name_mapper(name: str) -> str:
        name = name.lower()
        name = re.sub(r"(\(|\)|\[|\])", " ", name)
        name = re.sub(r"(\.|\\|\/|\,|\:|\;|\'|\")", " ", name)
        name = re.sub(r"\s+", "-", name.strip())
        name = re.sub(r"\-+", "-", name)
        name = re.sub(r"div\-(\d+)", r"div\1", name)
        name = re.sub(r"\-(\+|\>|\<)\-", r"\1", name)
        return name

    @staticmethod
    def name_type_mapper(name: str) -> NameTypes:
        name = name.lower()
        for name_type in NameTypes:
            if name == name_type.name.lower()[: len(name)]:
                return name_type
        # defaults to ALPHABETICAL
        return NameTypes.ALPHABETICAL

    @staticmethod
    def create_problem_name(idx: int, name_type: NameTypes) -> str:
        if name_type is NameTypes.ALPHABETICAL:
            return chr(ord("a") + idx)
        elif name_type is NameTypes.NUMERICAL:
            return f"{idx + 1}"
        elif name_type is NameTypes.ROMAN:
            # TODO
            return ""
        return ""



from ..helpers.model.finders.language_finder import LanguageFinder
from ..models.language import Language
from ..helpers.terminal.single_process import SingleProcess
from ..helpers.command.command_helper import CommandHelper
from ..executors.executor import Executor
from pylib_0xe.file.file import File


class PythonExecutor(Executor):
    @staticmethod
    def language() -> Language:
        return LanguageFinder.by_abbreviation("python")

    def execute(self):
        # run the problem and give input to it
        self.pipeline(self.run)

    def run(self):
        self.attach_log(f"running {self.problem.name}...")
        command = CommandHelper.python_run_command().format(self.problem.full_name)
        input_file = File.read_file(self.problem.full_input_name)
        input_file += "\r\n"
        out, err = SingleProcess(command).run().communicate(input_file)
        if len(err) > 0:
            # we have an error
            self.handle_error(err)
        self.handle_output(out)



from ..executors.cpp_executor import CppExecutor
from ..executors.python_executor import PythonExecutor
from ..executors.executor import Executor
from ..models.problem import Problem
from pylib_0xe.buffer_io.buffer_writer import BufferWriter


class ProgramExecutor:
    def __init__(self, problem: Problem, writer: BufferWriter) -> None:
        self.executor: Executor | None = None
        if problem.language == CppExecutor.language():
            self.executor = CppExecutor(problem, writer)
        if problem.language == PythonExecutor.language():
            self.executor = PythonExecutor(problem, writer)

    def exe(self):
        if self.executor is not None:
            self.executor.execute()



from dataclasses import dataclass, field
import os
from ..helpers.model.finders.language_finder import LanguageFinder
from ..helpers.model.finders.provider_finder import ProviderFinder
from ..helpers.model.problem_helper import ProblemHelper
from ..models.provider import Provider
from ..models.language import Language
from pylib_0xe.data.data_transfer_object import DataTransferObject


@dataclass
class Problem(DataTransferObject):
    """
    Represents a problem.
    """

    name: str = ""
    verdict: str = ""
    problem_set: str = ""
    provider: Provider = field(default_factory=ProviderFinder.default)
    language: Language = field(default_factory=LanguageFinder.default)
    path: str = "."

    @staticmethod
    def name_mapper(name: str) -> str:
        """
        Mapper function for problem name.
        """
        return ProblemHelper.problem_name_mapper(name)

    @staticmethod
    def provider_mapper(provider: str) -> Provider:
        """
        Mapper function for problem provider.
        """
        return ProviderFinder.by_abbreviation(provider)

    @staticmethod
    def language_mapper(language: str) -> Language:
        """
        Mapper function for problem language.
        """
        return LanguageFinder.by_abbreviation(language)

    @staticmethod
    def modifiables() -> list[str]:
        """
        Returns a list of modifiable attributes.
        """
        return ["name", "provider", "language"]

    @property
    def full_name(self) -> str:
        """
        Returns the full file name of the problem.
        """
        return os.path.join(self.path, f"{self.name}.{self.language.ext}")

    @property
    def full_input_name(self) -> str:
        """
        Returns the full file name of the input for the problem.
        """
        return os.path.join(self.path, f"{self.name}.in")

    def __str__(self) -> str:
        """
        Returns a string representation of the problem.
        """
        return self.full_name




from __future__ import annotations
from ..actions.problem_actions import ProblemActions
from ..executors.program_excecutor import ProgramExecutor
from ..models.problem import Problem
from pylib_0xe.buffer_io.standard_output_buffer import StandardOutputBuffer
from pylib_0xe.buffer_io.buffer_writer import BufferWriter
from pylib_0xe.argument.argument_parser import ArgumentParser


class ExecutionMediator:
    def read_configs(self) -> ExecutionMediator:
        return self

    def read_args(self) -> ExecutionMediator:
        value = ArgumentParser.get_value("run", option_prefix="--")
        if value is not None:
            # retrieving the problem based on the given name
            self.problem = Problem.from_dict({"path": value})
        else:
            # finding problem base on directory
            self.problem = Problem()
        return self

    def extract_language(self) -> ExecutionMediator:
        ProblemActions.set_problem_language(self.problem)
        return self

    def extract_name(self) -> ExecutionMediator:
        ProblemActions.set_problem_name(self.problem)
        return self

    def execute(self) -> ExecutionMediator:
        print(f'going to execute {self.problem}')
        ProgramExecutor(self.problem, BufferWriter(StandardOutputBuffer())).exe()
        return self



from abc import ABC, abstractmethod
from collections.abc import Callable
from dataclasses import dataclass
from ..models.language import Language
from ..helpers.utils.timer import Timer
from ..models.problem import Problem
from pylib_0xe.buffer_io.buffer_writer import BufferWriter


@dataclass
class Executor(ABC):
    problem: Problem
    writer: BufferWriter
    timer: Timer = Timer()
    status: int = 0

    @abstractmethod
    def execute(self):
        pass

    @staticmethod
    @abstractmethod
    def language() -> Language:
        pass

    def attach_log(self, log: str) -> None:
        self.writer.write_line(f"\t{log}")

    def pipeline(self, *args: Callable) -> None:
        for callable in args:
            if self.status == 0:
                self.timer = Timer()
                callable()
                self.attach_log(f"time elapsed = {self.timer.timestamp()}")

    def handle_output(self, output: str) -> None:
        if len(output) > 0:
            self.attach_log("<stdout")
            self.writer.write_line(output.strip())
            self.attach_log("stdout/>")
        else:
            self.attach_log("<stdout/>")

    def handle_error(self, error: str) -> None:
        self.status = 1
        self.attach_log("<stderr")
        self.writer.write_line(error)
        self.attach_log("stderr/>")



from dataclasses import dataclass, field
from ..helpers.model.finders.language_finder import LanguageFinder
from ..helpers.model.finders.provider_finder import ProviderFinder
from ..helpers.model.problem_helper import ProblemHelper
from ..models.language import Language
from ..models.provider import Provider
from ..types.name_types import NameTypes
from pylib_0xe.data.data_transfer_object import DataTransferObject


@dataclass
class Contest(DataTransferObject):
    """
    Represents a contest.
    """

    name_type: NameTypes = NameTypes.ALPHABETICAL
    name: str = ""
    problem_cnt: int = 0
    provider: Provider = field(default_factory=ProviderFinder.default)
    language: Language = field(default_factory=LanguageFinder.default)
    path: str = "."

    @staticmethod
    def modifiables() -> list[str]:
        """
        Returns a list of modifiable attributes.
        """
        return ["name", "provider", "language", "problem_cnt", "name_type"]

    @staticmethod
    def problem_cnt_mapper(number: int | str) -> int:
        """
        Mapper function for problem count.
        """
        return int(number)

    @staticmethod
    def name_type_mapper(key: str) -> NameTypes:
        """
        Mapper function for name types.
        """
        return ProblemHelper.name_type_mapper(key)

    @staticmethod
    def name_mapper(name: str) -> str:
        """
        Mapper function for contest name.
        """
        return ProblemHelper.problem_name_mapper(name)

    @staticmethod
    def provider_mapper(provider: str) -> Provider:
        """
        Mapper function for contest provider.
        """
        print(f"getting provider mapper {provider}")
        return ProviderFinder.by_abbreviation(provider)

    @staticmethod
    def language_mapper(language: str) -> Language:
        """
        Mapper function for contest language.
        """
        return LanguageFinder.by_abbreviation(language)
